#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"SimpleImage.h" /* 画像クラス */
#include"BMPIO.h" /* BMP画像の入出力関数 stdlib.hが必要 */

/* BMP画像を読み込んで正規化Gaussianフィルタを適用しBMP画像として保存する 
コンパイル: g++ -o GaussianFilter GaussianFilter.cxx -lm
実行： ./GaussianFilter 入力BMP 出力BMP 平均化する局所Windowの半径(int) ガウス関数の標準偏差sigmaパラメータ(double)
引数4
*/

/* 入力のR,G,Bに対してGaussianフィルタを適用する関数を作る: 出力はoutR,outG,outB、局所Windowの半径はradi、ガウス関数の標準偏差pパラメータはsigma、画像サイズはsx,sy */
void GaussianFilter(Image *R,Image *G,Image *B,Image *outR,Image *outG,Image *outB,int sx,int sy,int radi,double sigma){
  /* 正規化Gaussianフィルタここから 

注意点：実用的にはループ内部で毎回、特殊関数exp()を呼び出すのではなく、カーネル画像として局所Windowサイズのガウス画像を作り、そのガウス画像と入力の画像との積和(畳み込み、重み付き和)を計算する.
よって、プログラムが得意な人は、下記の構造で作らなくてもガウス画像を作る方法でもよい(+10点するので、レポートにはその旨を書く事).

*/  
  int i,j;
  /* 各画素をたどるループ */  
  for(i=0;i<sy;i++)
    for(j=0;j<sx;j++){
      /* R,G,B値を足し合わせるための変数を用意し、ゼロを代入する */
      
      /* 重みの正規化のための変数*/
      double weight_sum=0.0;
      
      int l,m;
      /* 局所Window内をたどるループ */  
      for(l=-radi;l<=radi;l++)
	for(m=-radi;m<=radi;m++){
	  /* 重みの変数 */
	  double weight;
	  /* 重みへガウス関数を代入する 
	     (正規化用の)ガウス関数: f(x,y) = exp(-(x^2+y^2)/(2sigma^2)) をプログラムになおしてweightへ代入.
	     ここでのx,yが何かをよく考える事.
	     また、^は二乗なので同じ変数をかける事(x^2はx*x).
	     計算は基本doubleなので2sigmaは2.0*simga
	  */
	  
	  
	  /* 画素の座標[i+l][j+m]のR,G,B値をweightを重みとして足し合わせる(重み付き和の計算) */
	  
	  
	  /* 後で正規化するためのに重みの総和を計算する */
	  weight_sum += weight;
	  
	  /* 注意点：画像の端では足さない、つまり(i+l)>=0,(i+l)<sy,(j+m)>=0,(j+m)<sxの条件のときだけ足し合わせる */
	}
      /* エラーチェック：理論的にはありえないが、重みの総和がゼロのときにゼロの割り算を防ぐ */
      if(weight_sum<=0.0)weight_sum=1.0;
      
      /* 出力のoutR,outG,outBへ足し合わせた値を重みの総和weight_sumで割って代入する */
      
      
    }
  
  
  /* 正規化Gaussianフィルタここまで */
  
  
}


int main(int argc,char *argv[]){
  
  
  /* 画像クラスImageを用いて入力画像のカラー用にR,G,Bを宣言する */
  
  

  /* 画像サイズ */
  int sx,sy;

  
  /* BMPIOクラスを宣言しnewする */
  
  /* BMPIOクラスのreadBMPSize()を使って、argv[1]で与えられたファイル名でsx,syに画像サイズを入れる */
  
  
  /* sx,syでR,G,Bのメモリを確保する(new する) */
  

  /* BMPIOクラスのreadBMP()を使ってargv[1]で与えられるファイル名の画像をR,G,Bに読み込む */

  
  /* 出力画像用にImageクラスをoutR,outG,outBとして宣言しsx,syのサイズでメモリを確保する */

   
  /* 平均化する局所Windowの半径を変数radiへセットする */
  int radi = atoi(argv[3]);
  
  /* ガウス関数の標準偏差sigmaパラメータをsigmaへセットする */
  double sigma = atof(argv[4]);
  
  /* Gaussianフィルタを関数を呼び出して適用する:GaussianFilter()の中身を作って下記の行のコメントアウトをはずす */
  //GaussianFilter(R,G,B,outR,outG,outB,sx,sy,radi,sigma);
  


  /* BMPIOクラスのsaveBMP()を使ってargv[2]で与えられるファイル名にoutR,outG,outBの画像を保存する */
  

  
  /* BMPIOクラスのメモリを開放する(deleteする) */
  

  /* R,G,Bのメモリを開放する(deleteする) */
  
  
  /* outR,outG,outBのメモリを開放する(deleteする) */
  
  

  return 0;
}
